import pytest

from colorviews import Color


class TestGetAttr:
    @pytest.mark.parametrize("attr, expected", [
        ("h", 0.75),
        ("s", 0.47),
        ("v", 0.29),
    ])
    def test_valid(self, attr, expected):
        color = Color.from_hsv(0.75, 0.47, 0.29)
        assert round(getattr(color.hsv, attr), 4) == expected

    @pytest.mark.parametrize("attr", [
        "r", "a",
    ])
    def test_invalid(self, attr):
        color = Color.from_hsv(0.75, 0.47, 0.29)
        with pytest.raises(AttributeError):
            getattr(color.hsv, attr)


class TestSetAttr:
    @pytest.mark.parametrize("attr, val", [
        ("h", 0.75),
        ("s", 0.5),
        ("v", 0.29),
    ])
    def test_valid(self, attr, val):
        color = Color.from_hsv(0.45, 0.15, 0.89)
        setattr(color.hsv, attr, val)
        assert round(getattr(color.hsv, attr), 4) == val

    @pytest.mark.parametrize("attr", [
        "r", "a",
    ])
    def test_invalid(self, attr):
        color = Color.from_hsv(0.75, 0.47, 0.29)
        with pytest.raises(AttributeError):
            setattr(color.hsv, attr, 0.1)


@pytest.mark.parametrize("hsv_dict, expected", [
    ({"h": 91 / 360}, 0x394a29),
    ({"s": 0.15}, 0x443f4a),
    ({"v": 0.74}, 0x9268bd),
    ({"h": 91 / 360, "s": 0.15}, 0x444a3f),
    ({"h": 91 / 360, "v": 0.74}, 0x91bd68),
    ({"s": 0.15, "v": 0.74}, 0xafa0bd),
    ({"h": 91 / 360, "s": 0.15, "v": 0.74}, 0xaebda0),
])
def test_replace(hsv_dict, expected):
    color = Color.from_hsv(0.75, 0.45, 0.29)
    assert int(color.hsv.replace(**hsv_dict)) == expected


class TestVals:
    @pytest.mark.parametrize("vals", [
        [0.2, 0.4, 0.6],
        (0.6, 0.2, 0.4),
    ])
    def test_setter_valid(self, vals):
        color = Color.from_hsv(0.75, 0.45, 0.29)
        color.hsv.vals = vals
        assert [round(val, 4) for val in color.hsv] == list(vals)

    @pytest.mark.parametrize("wrong_vals", [
        [0.2, 0.4],
        (0.6, 0.2, 0.4, 1.0),
        (1.6, 0.2, 0.4),
    ])
    def test_setter_invalid(self, wrong_vals):
        color = Color.from_hsv(0.75, 0.45, 0.29)
        with pytest.raises(ValueError):
            color.hsv.vals = wrong_vals


def test_vals_getter():
    color = Color.from_hsv(0.75, 0.45, 0.29)
    assert [round(val, 4) for val in color.hsv.vals] == [0.75, 0.45, 0.29]



import pytest

from colorviews import AlphaColor


class TestGetAttr:
    @pytest.mark.parametrize("attr, expected", [
        ("h", 0.75),
        ("s", 0.47),
        ("v", 0.29),
        ("a", 0.79),
    ])
    def test_valid(self, attr, expected):
        color = AlphaColor.from_hsva(0.75, 0.47, 0.29, 0.79)
        assert round(getattr(color.hsva, attr), 4) == expected

    @pytest.mark.parametrize("attr", [
        "r", "b",
    ])
    def test_invalid(self, attr):
        color = AlphaColor.from_hsva(0.75, 0.47, 0.29, 0.79)
        with pytest.raises(AttributeError):
            getattr(color.hsva, attr)


class TestSetAttr:
    @pytest.mark.parametrize("attr, val", [
        ("h", 0.75),
        ("s", 0.5),
        ("v", 0.29),
        ("a", 0.49),
    ])
    def test_valid(self, attr, val):
        color = AlphaColor.from_hsva(0.45, 0.15, 0.89, 0.79)
        setattr(color.hsva, attr, val)
        assert round(getattr(color.hsva, attr), 4) == val

    @pytest.mark.parametrize("attr", [
        "r", "g",
    ])
    def test_invalid(self, attr):
        color = AlphaColor.from_hsva(0.75, 0.47, 0.29, 0.79)
        with pytest.raises(AttributeError):
            setattr(color.hsva, attr, 0.1)


@pytest.mark.parametrize("hsva_dict, expected", [
    ({"h": 91 / 360}, 0x394a2980),
    ({"s": 0.15}, 0x443f4a80),
    ({"v": 0.74}, 0x9268bd80),
    ({"a": 0.80}, 0x39294acc),
    ({"h": 91 / 360, "s": 0.15}, 0x444a3f80),
    ({"h": 91 / 360, "v": 0.74}, 0x91bd6880),
    ({"h": 91 / 360, "v": 0.74, "a": 0.25}, 0x91bd6840),
    ({"s": 0.15, "v": 0.74}, 0xafa0bd80),
    ({"h": 91 / 360, "s": 0.15, "v": 0.74}, 0xaebda080),
])
def test_replace(hsva_dict, expected):
    color = AlphaColor.from_hsva(0.75, 0.45, 0.29, 0.5)
    assert int(color.hsva.replace(**hsva_dict)) == expected


class TestVals:
    @pytest.mark.parametrize("vals", [
        [0.2, 0.4, 0.6, 0.1],
        (0.6, 0.2, 0.4, 0.54),
    ])
    def test_setter_valid(self, vals):
        color = AlphaColor.from_hsva(0.75, 0.45, 0.29, 0.79)
        color.hsva.vals = vals
        assert [round(val, 4) for val in color.hsva] == list(vals)

    @pytest.mark.parametrize("wrong_vals", [
        [0.2, 0.4],
        (1.6, 0.2, 0.4),
        (0.6, 0.2, 0.4, 1.0, 0.8),
    ])
    def test_setter_invalid(self, wrong_vals):
        color = AlphaColor.from_hsva(0.75, 0.45, 0.29, 0.79)
        with pytest.raises(ValueError):
            color.hsva.vals = wrong_vals


def test_vals_getter():
    vals = (0.75, 0.45, 0.29, 0.79)
    color = AlphaColor.from_hsva(0.75, 0.45, 0.29, 0.79)
    assert [round(val, 4) for val in color.hsva.vals] == list(vals)



import pytest

from colorviews.utils import validate, scale


class TestValidate:
    @pytest.mark.parametrize("val", [
        0, 0.25, 1
    ])
    def test_valid(self, val):
        assert validate(val) is None

    @pytest.mark.parametrize("val", [
        -1, 1.2
    ])
    def test_invalid(self, val):
        with pytest.raises(ValueError):
            validate(val)


@pytest.mark.parametrize("val, factor, expected", [
    (0.5, 0xff, 128),
    (0.75, 100, 75),
    (0, 100, 0),
])
def test_scale(val, factor, expected):
    assert scale(val, factor) == expected



"""
Classes to represent colors with and without alpha value.

Color represents color values without alpha and AlphaColor represents color
values that have alpha.
"""

import colorsys

import colorviews.names
import colorviews.views
import colorviews.utils as utils


class BaseColor:
    """
    Base class of all Color classes.
    """

    def __init__(self,
                 r: float,
                 g: float,
                 b: float):
        utils.validate(r)
        utils.validate(g)
        utils.validate(b)
        self._r = r
        self._g = g
        self._b = b

    def __int__(self):
        red = utils.scale(self._r, 0xff)
        green = utils.scale(self._g, 0xff)
        blue = utils.scale(self._b, 0xff)
        intval = ((red << 16) | (green << 8) | blue)
        return intval

    def __str__(self):
        # integer color value in hex form as a string with any prefix
        return f"{int(self):x}"

    def __repr__(self):
        cls_name = self.__class__.__name__
        return f"<{cls_name}(0x{int(self):x})>"

    def __index__(self):
        # for hex()
        return int(self)

    def __lt__(self, other: object):
        if isinstance(other, self.__class__):
            return int(self) < int(other)
        return NotImplemented

    def __le__(self, other: object):
        if isinstance(other, self.__class__):
            return int(self) <= int(other)
        return NotImplemented

    def __eq__(self, other: object):
        if isinstance(other, self.__class__):
            return int(self) == int(other)
        return NotImplemented

    def __ne__(self, other: object):
        if isinstance(other, self.__class__):
            return int(self) != int(other)
        return NotImplemented

    def __gt__(self, other: object):
        if isinstance(other, self.__class__):
            return int(self) > int(other)
        return NotImplemented

    def __ge__(self, other: object):
        if isinstance(other, self.__class__):
            return int(self) >= int(other)
        return NotImplemented


class Color(BaseColor):
    """
    Class representing colors without alpha value.

    Information is stored using rgb values.
    """
    @property
    def rgb(self):
        """RGB colorview as float values"""
        return colorviews.views.ColorViewRGB(self)

    @property
    def hsl(self):
        """HSL colorview as float values"""
        return colorviews.views.ColorViewHSL(self)

    @property
    def hsv(self):
        """HSV colorview as float values"""
        return colorviews.views.ColorViewHSV(self)

    @classmethod
    def from_name(cls, name: str) -> 'Color':
        """
        Creates a Color object based on the given color name.

        Only CSS3 extended color keyword names are recognized.

        Arguments:
          name: Name of color.

        Returns:
          Color object with value corresponding to the given color name.
        """
        name = name.lower()
        return cls.from_int(colorviews.names.COLORS[name])

    @classmethod
    def from_int(cls, value: int):
        """
        Creates a Color object from the given integer color value.

        Arguments:
          value: Integer color value.

        Returns:
          Color object with given integer value.

        Raises:
          ValueError: If value is outside the valid range.
        """
        if value < 0 or value > 0xffffff:
            raise ValueError(f"{value}: invalid color value")
        red = ((value >> 16) & 0xff) / 0xff
        green = ((value >> 8) & 0xff) / 0xff
        blue = (value & 0xff) / 0xff
        return cls(red, green, blue)

    @classmethod
    def from_rgb(cls, r: float, g: float, b: float) -> "Color":
        """
        Creates a Color object from the given RGB float values.

        Arguments:
          r: Red component of RGB value as a float in the range [0.0, 1.0]
          g: Green component of RGB value as a float in the range [0.0, 1.0]
          b: Blue component of RGB value as a float in the range [0.0, 1.0]

        Returns:
          Color object with given RGB values.

        Raises:
          ValueError: If value is outside the valid range.
        """
        return cls(r, g, b)

    @classmethod
    def from_hsl(cls, h: float, s: float, l: float) -> "Color":
        """
        Creates a Color object from the given HSL float values.

        Arguments:
          h: Hue component of HSL value as a float in the range [0.0, 1.0]
          s: Saturation component of HSL value as a float in the
             range [0.0, 1.0]
          l: Lightness component of HSL value as a float in the
             range [0.0, 1.0]

        Returns:
          Color object with given HSL values.

        Raises:
          ValueError: If any argument is outside valid range.
        """
        utils.validate(h)
        utils.validate(s)
        utils.validate(l)
        rgb = colorsys.hls_to_rgb(h, l, s)
        return cls(*rgb)

    @classmethod
    def from_hsv(cls, h: float, s: float, v: float) -> "Color":
        """
        Creates a Color object from the given HSV float values.

        Arguments:
          h: Hue component of HSV value as a float in the range [0.0, 1.0]
          s: Saturation component of HSV value as a float in the
             range [0.0, 1.0]
          v: Value component of HSV value as a float in the
             range [0.0, 1.0]

        Returns:
          Color object with given HSV values.

        Raises:
          ValueError: If any argument is outside valid range.
        """
        utils.validate(h)
        utils.validate(s)
        utils.validate(v)
        rgb = colorsys.hsv_to_rgb(h, s, v)
        return cls(*rgb)

    def copy(self) -> "Color":
        """
        Creates a copy of the Color object.

        Returns:
          An identical copy of the Color object.
        """
        return Color(self._r, self._g, self._b)


class AlphaColor(BaseColor):
    """
    Class representing colors with alpha value.

    Information is stored using rgba values.
    """
    def __init__(self,
                 r: float,
                 g: float,
                 b: float,
                 a: float):
        super().__init__(r, g, b)
        utils.validate(a)
        self._a = a

    def __int__(self):
        alpha = utils.scale(self._a, 0xff)
        nonalphaint = super().__int__()
        intval = (nonalphaint << 8) | alpha
        return intval

    @property
    def rgba(self):
        """RGBA colorview as float values"""
        return colorviews.views.ColorViewRGBA(self)

    @property
    def hsla(self):
        """HSLA colorview as float values"""
        return colorviews.views.ColorViewHSLA(self)

    @property
    def hsva(self):
        """HSVA colorview as float values"""
        return colorviews.views.ColorViewHSVA(self)

    @classmethod
    def from_name(cls, name: str, a: float = 0.0) -> "AlphaColor":
        """
        Creates an AlphaColor object based on the given color name.

        Only CSS3 extended color keyword names are recognized.

        Arguments:
          name: Name of color.
          a: Alpha value of color as a float in the range [0.0, 1.0].
             Default value is 0.0

        Returns:
          AlphaColor object with value corresponding to the given color name
          and alpha value.
        """
        utils.validate(a)
        a_int = int(a * 0xff)
        name = name.lower()
        rgb_intval = colorviews.names.COLORS[name]
        rgba_intval = (rgb_intval << 8) | a_int
        return cls.from_int(rgba_intval)

    @classmethod
    def from_int(cls, value: int) -> "AlphaColor":
        """
        Creates an AlphaColor object from the given integer color value.

        Arguments:
          value: Integer color value.

        Returns:
          AlphaColor object with given integer value.

        Raises:
          ValueError: If value is outside the valid range.
        """
        if value < 0 or value > 0xffffffff:
            raise ValueError(f"{value}: invalid alphacolor value")
        red = ((value >> 24) & 0xff) / 0xff
        green = ((value >> 16) & 0xff) / 0xff
        blue = ((value >> 8) & 0xff) / 0xff
        alpha = (value & 0xff) / 0xff
        return cls(red, green, blue, alpha)

    @classmethod
    def from_rgba(cls, r: float, g: float, b: float, a: float) -> "AlphaColor":
        """
        Creates an AlphaColor object from the given RGBA float values.

        Arguments:
          r: Red component of RGBA value as a float in the range [0.0, 1.0].
          g: Green component of RGBA value as a float in the range [0.0, 1.0].
          b: Blue component of RGBA value as a float in the range [0.0, 1.0].
          a: Alpha component of RGBA value as a float in the range [0.0, 1.0].

        Returns:
          AlphaColor object with given RGBA values.

        Raises:
          ValueError: If value is outside the valid range.
        """
        return cls(r, g, b, a)

    @classmethod
    def from_hsla(cls, h: float, s: float, l: float, a: float) -> "AlphaColor":
        """
        Creates an AlphaColor object from the given HSLA float values.

        Arguments:
          h: Hue component of HSLA value as a float in the range [0.0, 1.0]
          s: Saturation component of HSLA value as a float in the
             range [0.0, 1.0].
          l: Lightness component of HSLA value as a float in the
             range [0.0, 1.0].
          a: Alpha component of HSLA value as a float in the range [0.0, 1.0].

        Returns:
          AlphaColor object with given HSLA values.

        Raises:
          ValueError: If value is outside the valid range.
        """
        utils.validate(h)
        utils.validate(s)
        utils.validate(l)
        utils.validate(a)
        rgba = colorsys.hls_to_rgb(h, l, s) + (a, )
        return cls(*rgba)

    @classmethod
    def from_hsva(cls, h: float, s: float, v: float, a: float) -> "AlphaColor":
        """
        Creates an AlphaColor object from the given HSVA float values.

        Arguments:
          h: Hue component of HSVA value as a float in the range [0.0, 1.0]
          s: Saturation component of HSVA value as a float in the
             range [0.0, 1.0].
          v: Value component of HSVA value as a float in the
             range [0.0, 1.0].
          a: Alpha component of HSVA value as a float in the range [0.0, 1.0].

        Returns:
          AlphaColor object with given HSVA values.

        Raises:
          ValueError: If value is outside the valid range.
        """
        utils.validate(h)
        utils.validate(s)
        utils.validate(v)
        utils.validate(a)
        rgba = colorsys.hsv_to_rgb(h, s, v) + (a, )
        return cls(*rgba)

    def copy(self) -> "AlphaColor":
        """
        Creates a copy of the AlphaColor object.

        Returns:
          An identical copy of the AlphaColor object.
        """
        return AlphaColor(self._r, self._g, self._b, self._a)



"""
colorviews module
"""

__all__ = ["Color", "AlphaColor", "scale"]
__version__ = "0.1-alpha1"
__author__ = "Julin Shaji"

from colorviews.colors import Color, AlphaColor
from colorviews.utils import scale



import pytest

from colorviews import Color


class TestGetAttr:
    @pytest.mark.parametrize("attr, expected", [
        ("h", 0.75),
        ("s", 0.47),
        ("l", 0.29),
    ])
    def test_valid(self, attr, expected):
        color = Color.from_hsl(0.75, 0.47, 0.29)
        assert round(getattr(color.hsl, attr), 4) == expected

    @pytest.mark.parametrize("attr", [
        "r", "a",
    ])
    def test_invalid(self, attr):
        color = Color.from_hsl(0.75, 0.47, 0.29)
        with pytest.raises(AttributeError):
            getattr(color.hsl, attr)


class TestSetAttr:
    @pytest.mark.parametrize("attr, val", [
        ("h", 0.75),
        ("s", 0.5),
        ("l", 0.29),
    ])
    def test_valid(self, attr, val):
        color = Color.from_hsl(0.45, 0.15, 0.89)
        setattr(color.hsl, attr, val)
        assert round(getattr(color.hsl, attr), 4) == val

    @pytest.mark.parametrize("attr", [
        "r", "a",
    ])
    def test_invalid(self, attr):
        color = Color.from_hsl(0.75, 0.47, 0.29)
        with pytest.raises(AttributeError):
            setattr(color.hsl, attr, 0.1)


@pytest.mark.parametrize("hsl_dict, expected", [
    ({"h": 91 / 360}, 0x496b29),
    ({"s": 0.15}, 0x4a3f55),
    ({"l": 0.74}, 0xbd9fdb),
    ({"h": 91 / 360, "s": 0.15}, 0x4a553f),
    ({"h": 91 / 360, "l": 0.74}, 0xbcdb9f),
    ({"s": 0.15, "l": 0.74}, 0xbdb3c7),
    ({"h": 91 / 360, "s": 0.15, "l": 0.74}, 0xbcc7b3),
])
def test_replace(hsl_dict, expected):
    color = Color.from_hsl(0.75, 0.45, 0.29)
    assert int(color.hsl.replace(**hsl_dict)) == expected


class TestVals:
    @pytest.mark.parametrize("vals", [
        [0.2, 0.4, 0.6],
        (0.6, 0.2, 0.4),
    ])
    def test_setter_valid(self, vals):
        color = Color.from_hsl(0.75, 0.45, 0.29)
        color.hsl.vals = vals
        assert [round(val, 4) for val in color.hsl] == list(vals)

    @pytest.mark.parametrize("wrong_vals", [
        [0.2, 0.4],
        (0.6, 0.2, 0.4, 1.0),
        (1.6, 0.2, 0.4),
    ])
    def test_setter_invalid(self, wrong_vals):
        color = Color.from_hsl(0.75, 0.45, 0.29)
        with pytest.raises(ValueError):
            color.hsl.vals = wrong_vals


def test_vals_getter():
    color = Color.from_hsl(0.75, 0.45, 0.29)
    assert [round(val, 4) for val in color.hsl.vals] == [0.75, 0.45, 0.29]



"""
Utility functions to help other modules.
"""


def validate(val: float) -> None:
    """
    Check if val is between 0.0 and 1.0, including the limits.

    Arguments:
      val: Value to be validated.

    Raises:
      ValueError: If val is not in the range [0.0, 1.0].
    """
    if val < 0 or val > 1:
        raise ValueError(f"{val}: Expected value in [0.0, 1.0]")


def scale(val: float, factor: int) -> int:
    """
    Multiply val with factor and convert result to an integer with proper
    rounding.

    Arguments:
      val: Float value to rounded
      factor: Value with which val is multiplied.

    Returns:
      Rounded integer value of the product of val and factor.
    """
    num = val * factor
    floor_int = int(num)
    if num - floor_int < 0.5:
        return floor_int
    return floor_int + 1



import pytest

from colorviews import Color


class TestGetAttr:
    @pytest.mark.parametrize("attr, expected", [
        ("r", 0.75),
        ("g", 0.45),
        ("b", 0.29),
    ])
    def test_valid(self, attr, expected):
        color = Color.from_rgb(0.75, 0.45, 0.29)
        assert round(getattr(color.rgb, attr), 4) == expected

    @pytest.mark.parametrize("attr", [
        "s", "a",
    ])
    def test_invalid(self, attr):
        color = Color.from_rgb(0.75, 0.45, 0.29)
        with pytest.raises(AttributeError):
            getattr(color.rgb, attr)


class TestSetAttr:
    @pytest.mark.parametrize("attr, val", [
        ("r", 0.45),
        ("g", 0.15),
        ("b", 0.89),
    ])
    def test_valid(self, attr, val):
        color = Color.from_rgb(0.75, 0.45, 0.29)
        setattr(color.rgb, attr, val)
        assert round(getattr(color.rgb, attr), 4) == val

    def test_vals(self):
        color = Color.from_rgb(0.75, 0.45, 0.29)
        color.rgb.vals = (0.2, 0.4, 0.6)
        assert color.rgb.vals == (0.2, 0.4, 0.6)

    @pytest.mark.parametrize("attr", [
        "s", "a",
    ])
    def test_invalid(self, attr):
        color = Color.from_rgb(0.75, 0.45, 0.29)
        with pytest.raises(AttributeError):
            setattr(color.rgb, attr, 0.1)


@pytest.mark.parametrize("rgb_dict, expected", [
    ({"r": 91 / 0xff}, 0x5b734a),
    ({"g": 15 / 0xff}, 0xbf0f4a),
    ({"b": 174 / 0xff}, 0xbf73ae),
    ({"r": 91 / 0xff, "g": 15 / 0xff}, 0x5b0f4a),
    ({"r": 91 / 0xff, "b": 174 / 0xff}, 0x5b73ae),
    ({"g": 15 / 0xff, "b": 174 / 0xff}, 0xbf0fae),
    ({"r": 91 / 0xff, "g": 15 / 0xff, "b": 174 / 0xff}, 0x5b0fae),
])
def test_replace(rgb_dict, expected):
    color = Color.from_rgb(0.75, 0.45, 0.29)  # rgb(191,115,74)
    assert int(color.rgb.replace(**rgb_dict)) == expected


class TestVals:
    @pytest.mark.parametrize("vals", [
        [0.2, 0.4, 0.6],
        (0.6, 0.2, 0.4),
    ])
    def test_setter_valid(self, vals):
        color = Color.from_rgb(0.75, 0.45, 0.29)  # rgb(191,115,74)
        color.rgb.vals = vals
        assert list(color.rgb) == list(vals)

    @pytest.mark.parametrize("wrong_vals", [
        [0.2, 0.4],
        (0.6, 0.2, 0.4, 1.0),
        (1.6, 0.2, 0.4),
    ])
    def test_setter_invalid(self, wrong_vals):
        color = Color.from_rgb(0.75, 0.45, 0.29)  # rgb(191,115,74)
        with pytest.raises(ValueError):
            color.rgb.vals = wrong_vals

    def test_getter(self):
        color = Color.from_rgb(0.75, 0.45, 0.29)
        assert color.rgb.vals == (0.75, 0.45, 0.29)



import pytest

from colorviews import AlphaColor


class TestGetAttr:
    @pytest.mark.parametrize("attr, expected", [
        ("r", 0.75),
        ("g", 0.45),
        ("b", 0.29),
        ("a", 0.79),
    ])
    def test_valid(self, attr, expected):
        color = AlphaColor.from_rgba(0.75, 0.45, 0.29, 0.79)
        assert round(getattr(color.rgba, attr), 4) == expected

    def test_vals(self):
        color = AlphaColor.from_rgba(0.75, 0.45, 0.29, 0.81)
        assert color.rgba.vals == (0.75, 0.45, 0.29, 0.81)

    @pytest.mark.parametrize("attr", [
        "s", "v",
    ])
    def test_invalid(self, attr):
        color = AlphaColor.from_rgba(0.75, 0.45, 0.29, 0.81)
        with pytest.raises(AttributeError):
            getattr(color.rgba, attr)


class TestSetAttr:
    @pytest.mark.parametrize("attr, val", [
        ("r", 0.45),
        ("g", 0.15),
        ("b", 0.89),
        ("a", 0.49),
    ])
    def test_valid(self, attr, val):
        color = AlphaColor.from_rgba(0.75, 0.45, 0.29, 0.75)
        setattr(color.rgba, attr, val)
        assert round(getattr(color.rgba, attr), 4) == val

    @pytest.mark.parametrize("attr", [
        "s", "v",
    ])
    def test_invalid(self, attr):
        color = AlphaColor.from_rgba(0.75, 0.45, 0.29, 0.75)
        with pytest.raises(AttributeError):
            setattr(color.rgba, attr, 0.1)


@pytest.mark.parametrize("rgba_dict, expected", [
    ({"r": 0x5b / 0xff}, 0x5b734a80),
    ({"g": 0x0f / 0xff}, 0xbf0f4a80),
    ({"b": 0xae / 0xff}, 0xbf73ae80),
    ({"a": 0.23}, 0xbf734a3b),
    ({"r": 0x5b / 0xff, "g": 0x0f / 0xff}, 0x5b0f4a80),
    ({"r": 0x5b / 0xff, "b": 0xae / 0xff}, 0x5b73ae80),
    ({"g": 0x0f / 0xff, "b": 0xae / 0xff, "a": 0.52}, 0xbf0fae85),
    ({"r": 0xff / 0xff, "g": 0x0f / 0xff, "a": 0.52}, 0xff0f4a85),
])
def test_replace(rgba_dict, expected):
    color = AlphaColor.from_int(0xbf734a80)  # rgba(191,115,74,50%)
    assert int(color.rgba.replace(**rgba_dict)) == expected


class TestVals:
    def test_getter(self):
        color = AlphaColor.from_rgba(0.75, 0.45, 0.29, 0.75)
        color.rgba.vals = (0.2, 0.4, 0.6, 0.23)
        assert color.rgba.vals == (0.2, 0.4, 0.6, 0.23)

    @pytest.mark.parametrize("vals", [
        [0.2, 0.4, 0.6, 0.23],
        (0.6, 0.2, 0.4, 0.93),
    ])
    def test_setter_valid(self, vals):
        color = AlphaColor.from_rgba(0.75, 0.45, 0.29, 0.2)  # rgba(191,115,74)
        color.rgba.vals = vals
        assert list(color.rgba) == list(vals)

    @pytest.mark.parametrize("wrong_vals", [
        [0.2, 0.4],
        (0.6, 0.2, 0.4),
        (0.6, 0.2, 0.4, 1.2),
        (0.6, 0.2, 0.4, 1.0, 0.2),
    ])
    def test_setter_invalid(self, wrong_vals):
        color = AlphaColor.from_rgba(0.75, 0.45, 0.29, 0.1)  # rgba(191,115,74)
        with pytest.raises(ValueError):
            color.rgba.vals = wrong_vals



import pytest
from colorviews import Color, AlphaColor


class TestComparisons:
    def test_eq(self):
        color_a = Color.from_int(0xefab3)
        color_b = Color.from_int(0xefab3)
        assert color_a == color_b

    def test_ne(self):
        color_a = Color.from_int(0xefab3)
        color_b = Color.from_int(0xffab3)
        assert color_a != color_b

    def test_lt(self):
        color_a = Color.from_int(0xefab3)
        color_b = Color.from_int(0xffab3)
        assert color_a < color_b

    def test_le(self):
        color_a = Color.from_int(0xefab3)
        color_b = Color.from_int(0xefab3)
        assert color_a <= color_b

    def test_gt(self):
        color_a = Color.from_int(0xffab3)
        color_b = Color.from_int(0xefab3)
        assert color_a > color_b

    def test_ge(self):
        color_a = Color.from_int(0xefab3)
        color_b = Color.from_int(0xefab3)
        assert color_a >= color_b

    @pytest.mark.parametrize("func,arg", [
        ("__lt__", 12),
        ("__le__", "string"),
        ("__gt__", 12),
        ("__ge__", 12),
        ("__eq__", 12),
        ("__ne__", 1.2),
    ])
    def test_invalid_comparison(self, func, arg):
        color = Color.from_int(0xefab3)
        assert getattr(color, func)(arg) == NotImplemented

    @pytest.mark.parametrize("func", [
        "__lt__",
        "__le__",
        "__gt__",
        "__ge__",
        "__eq__",
        "__ne__",
    ])
    def test_alphacolor_comparison(self, func):
        color = Color.from_int(0xefab3)
        alphacolor = AlphaColor.from_name("red")
        assert getattr(color, func)(alphacolor) == NotImplemented


class TestFromInt:
    @pytest.mark.parametrize("colorint, rgb", [
        (0x5b734a, [0.3569, 0.4510, 0.2902]),
        (0xbf0f4a, [0.7490, 0.0588, 0.2902]),
    ])
    def test_valid(self, colorint, rgb):
        color = Color.from_int(colorint)
        rounded_comps = [round(comp, 4) for comp in color.rgb]
        assert rounded_comps == rgb

    @pytest.mark.parametrize("colorint", [
        0x1fabcdef,
        -234,
    ])
    def test_invalid(self, colorint):
        with pytest.raises(ValueError):
            Color.from_int(colorint)


class TestFromRGB:
    def test_valid(self):
        assert int(Color.from_rgb(0.6, 0.2, 0.8)) == 0x9933cc

    def test_invalid(self):
        with pytest.raises(ValueError):
            Color.from_rgb(0.6, 0.2, 0xcc)


class TestFromHSL:
    def test_valid(self):
        assert int(Color.from_hsl(0.625, 0.15, 0.74)) == 0xb3b8c7

    def test_invalid(self):
        with pytest.raises(ValueError):
            Color.from_hsl(25, 1.0, 0.74)


class TestFromHSV:
    def test_valid(self):
        assert int(Color.from_hsv(0.625, 0.15, 0.74)) == 0xa0a7bd

    def test_invalid(self):
        with pytest.raises(ValueError):
            Color.from_hsv(25, 1.0, 0.74)


def test_str():
    color = Color.from_int(0xabcdef)
    assert str(color) == "abcdef"


def test_repr():
    color = Color.from_int(0xabcdef)
    assert repr(color) == "<Color(0xabcdef)>"


def test_hex():
    color = Color.from_int(0xabcdef)
    assert hex(color) == "0xabcdef"


def test_from_name():
    color = Color.from_name("gray")
    assert int(color) == 0x808080


def test_from_rgb():
    assert int(Color.from_rgb(0.3569, 0.4510, 0.2902)) == 0x5b734a


def test_from_hsl():
    assert int(Color.from_hsl(0.2583, 0.1515, 0.7412)) == 0xbcc7b3


def test_copy():
    orig = Color.from_hsl(0.2583, 0.1515, 0.7412)
    copy = orig.copy()
    assert id(orig) != id(copy)
    assert orig.rgb.vals == copy.rgb.vals



import pathlib
import setuptools

long_description = pathlib.Path("README.md").read_text()

setuptools.setup(
    name="colorviews",
    version="0.1-alpha1",
    author="Julin S",
    author_email="",
    description="A simple module to handle colors",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ju-sh/colorviews",
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    classifiers=[
        # https://pypi.org/pypi?%3Aaction=list_classifiers
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    project_urls={
        'Changelog': 'https://github.com/ju-sh/colorviews/blob/master/CHANGELOG.md',
        'Issue Tracker': 'https://github.com/ju-sh/colorviews/issues',
    },
    install_requires=[],
    python_requires='>=3.6',
)



"""
ColorView classes for Color and AlphaColor objects

For Color:

 - ColorViewRGB
 - ColorViewHSL
 - ColorViewHSV

For AlphaColor:

 - ColorViewRGBA
 - ColorViewHSLA
 - ColorViewHSVA
"""

import colorsys

from typing import cast, List, Sequence, TYPE_CHECKING
import colorviews.utils as utils

if TYPE_CHECKING:
    import colorviews.colors as colors


class BaseColorView:
    """
    Base class of all ColorView classes.
    """
    __slots__ = ["color"]

    def __init__(self, color: "colors.BaseColor"):
        self.color = color


class BaseAlphaColorView(BaseColorView):
    """
    Base class of all ColorView classes with alpha value.
    """
    __slots__: List[str] = []

    @property
    def a(self) -> float:
        """Alpha component as a float value"""
        self.color = cast("colors.AlphaColor", self.color)
        return self.color._a

    @a.setter
    def a(self, val: float):
        utils.validate(val)
        self.color = cast("colors.AlphaColor", self.color)
        self.color._a = val


class BaseColorViewRGB(BaseColorView):
    """
    Base class of ColorViewRGB and ColorViewRGBA classes.
    """
    __slots__: List[str] = []

    @property
    def r(self) -> float:
        """Red component as a float value"""
        return self.color._r

    @r.setter
    def r(self, val: float):
        utils.validate(val)
        self.color._r = val

    @property
    def g(self) -> float:
        """Green component as a float value"""
        return self.color._g

    @g.setter
    def g(self, val: float):
        utils.validate(val)
        self.color._g = val

    @property
    def b(self) -> float:
        """Blue component as a float value"""
        return self.color._b

    @b.setter
    def b(self, val: float):
        utils.validate(val)
        self.color._b = val


class ColorViewRGB(BaseColorViewRGB):
    """
    RGB view of Color objects in float values
    """
    __slots__: List[str] = []

    @property
    def vals(self) -> Sequence[float]:
        """
        Tuple of RGB values as floats.
        """
        return tuple(self)

    @vals.setter
    def vals(self, values: Sequence[float]):
        if len(values) != 3:
            raise ValueError("Needs exactly 3 floats")
        utils.validate(values[0])
        utils.validate(values[1])
        utils.validate(values[2])
        self.r, self.g, self.b = values

    def __iter__(self):
        yield from [self.r, self.g, self.b]

    def replace(self, r=None, g=None, b=None) -> "colors.Color":
        """
        Create a new Color object by replacing the RGB values of the Color
        object associated with the colorview.

        Arguments:
          r: Red component of RGB value as a float in the range [0, 0.1].
          g: Green component of RGB value as a float in the range [0, 0.1].
          b: Blue component of RGB value as a float in the range [0, 0.1].

        Returns:
          Color object with modified RGB values.
        """
        if r is None:
            r = self.r
        if g is None:
            g = self.g
        if b is None:
            b = self.b
        self.color = cast("colors.Color", self.color)
        return self.color.from_rgb(r, g, b)


class ColorViewRGBA(BaseColorViewRGB, BaseAlphaColorView):
    """
    RGBA view of Color objects in float values
    """
    __slots__: List[str] = []

    @property
    def vals(self) -> Sequence[float]:
        """
        Tuple of RGBA values as floats.
        """
        return tuple(self)

    @vals.setter
    def vals(self, values: Sequence[float]):
        if len(values) != 4:
            raise ValueError("Needs exactly 4 floats")
        utils.validate(values[0])
        utils.validate(values[1])
        utils.validate(values[2])
        utils.validate(values[3])
        self.r, self.g, self.b, self.a = values

    def __iter__(self):
        yield from [self.r, self.g, self.b, self.a]

    def replace(self, r=None, g=None, b=None, a=None) -> "colors.AlphaColor":
        """
        Create a new Color object by replacing the RGBA values of the Color
        object associated with the colorview.

        Arguments:
          r: Red component of RGBA value as a float in the range [0, 0.1].
          g: Green component of RGBA value as a float in the range [0, 0.1].
          b: Blue component of RGBA value as a float in the range [0, 0.1].
          a: Alpha component of RGBA value as a float in the range [0, 0.1].

        Returns:
          Color object with modified RGBA values.
        """
        if r is None:
            r = self.r
        if g is None:
            g = self.g
        if b is None:
            b = self.b
        if a is None:
            a = self.a
        self.color = cast("colors.AlphaColor", self.color)
        return self.color.from_rgba(r, g, b, a)


class BaseColorViewHSL(BaseColorView):
    """
    Base class of ColorViewHSL and ColorViewHSLA classes.
    """
    __slots__: List[str] = []

    @property
    def h(self):
        """Hue component as a float value"""
        rgb = (self.color._r, self.color._g, self.color._b)
        hls = colorsys.rgb_to_hls(*rgb)
        return hls[0]

    @h.setter
    def h(self, val: float):
        utils.validate(val)
        rgb = (self.color._r, self.color._g, self.color._b)
        hls = colorsys.rgb_to_hls(*rgb)
        rgb = colorsys.hls_to_rgb(val, hls[1], hls[2])
        self.color._r, self.color._g, self.color._b = rgb

    @property
    def s(self):
        """Saturation component as a float value"""
        rgb = (self.color._r, self.color._g, self.color._b)
        hls = colorsys.rgb_to_hls(*rgb)
        return hls[2]

    @s.setter
    def s(self, val: float):
        utils.validate(val)
        rgb = (self.color._r, self.color._g, self.color._b)
        hls = colorsys.rgb_to_hls(*rgb)
        rgb = colorsys.hls_to_rgb(hls[0], hls[2], val)
        self.color._r, self.color._g, self.color._b = rgb

    @property
    def l(self):
        """Lightness component as a float value"""
        rgb = (self.color._r, self.color._g, self.color._b)
        hls = colorsys.rgb_to_hls(*rgb)
        return hls[1]

    @l.setter
    def l(self, val: float):
        utils.validate(val)
        rgb = (self.color._r, self.color._g, self.color._b)
        hls = colorsys.rgb_to_hls(*rgb)
        rgb = colorsys.hls_to_rgb(hls[0], val, hls[1])
        self.color._r, self.color._g, self.color._b = rgb


class ColorViewHSL(BaseColorViewHSL):
    """
    HSL view of Color objects in float values
    """
    __slots__: List[str] = []

    def __iter__(self):
        hls = colorsys.rgb_to_hls(*self.color.rgb.vals)
        yield from [hls[0], hls[2], hls[1]]

    @property
    def vals(self) -> Sequence[float]:
        """
        Tuple of HSL values as floats.
        """
        return tuple(self)

    @vals.setter
    def vals(self, values: Sequence[float]):
        if len(values) != 3:
            raise ValueError("Needs exactly 3 floats")
        utils.validate(values[0])
        utils.validate(values[1])
        utils.validate(values[2])
        rgb = colorsys.hls_to_rgb(values[0], values[2], values[1])
        self.color = cast("colors.Color", self.color)
        self.color.rgb.vals = rgb

    def replace(self, h=None, s=None, l=None) -> "colors.Color":
        """
        Create a new Color object by replacing the HSL values of the Color
        object associated with the colorview.

        Arguments:
          h: Hue component of HSL value as a float in the range [0, 0.1].
          s: Saturation component of HSL value as a float in the
             range [0, 0.1].
          l: Lightness component of HSL value as a float in the range [0, 0.1].

        Returns:
          Color object with modified HSL values.
        """
        hsl = list(self)
        if h is None:
            h = hsl[0]
        if s is None:
            s = hsl[1]
        if l is None:
            l = hsl[2]
        self.color = cast("colors.Color", self.color)
        return self.color.from_hsl(h, s, l)


class ColorViewHSLA(BaseColorViewHSL, BaseAlphaColorView):
    """
    HSLA view of Color objects in float values
    """
    __slots__: List[str] = []

    def __iter__(self):
        yield from [self.h, self.s, self.l, self.a]

    @property
    def vals(self) -> Sequence[float]:
        """
        Tuple of HSLA values as floats.
        """
        return tuple(self)

    @vals.setter
    def vals(self, values: Sequence[float]):
        if len(values) != 4:
            raise ValueError("Needs exactly 4 floats")
        utils.validate(values[0])
        utils.validate(values[1])
        utils.validate(values[2])
        utils.validate(values[3])
        rgb = colorsys.hls_to_rgb(values[0], values[2], values[1])
        self.color = cast("colors.AlphaColor", self.color)
        self.color.rgba.vals = rgb + (values[3], )

    def replace(self, h=None, s=None, l=None, a=None) -> "colors.AlphaColor":
        """
        Create a new AlphaColor object by replacing the HSLA values of the
        AlphaColor object associated with the colorview.

        Arguments:
          h: Hue component of HSLA value as a float in the range [0, 0.1].
          s: Saturation component of HSLA value as a float in the
             range [0, 0.1].
          l: Lightness component of HSLA value as a float in the
             range [0, 0.1].
          a: Alpha component of HSLA value as a float in the range [0, 0.1].

        Returns:
          AlphaColor object with modified HSLA values.
        """
        hsla = list(self)
        if h is None:
            h = hsla[0]
        if s is None:
            s = hsla[1]
        if l is None:
            l = hsla[2]
        if a is None:
            a = hsla[3]
        self.color = cast("colors.AlphaColor", self.color)
        return self.color.from_hsla(h, s, l, a)


class BaseColorViewHSV(BaseColorView):
    """
    Base class of ColorViewHSV and ColorViewHSVA classes.
    """
    __slots__: List[str] = []

    @property
    def h(self):
        """Hue component as a float value"""
        rgb = (self.color._r, self.color._g, self.color._b)
        hsv = colorsys.rgb_to_hsv(*rgb)
        return hsv[0]

    @h.setter
    def h(self, val: float):
        utils.validate(val)
        rgb = (self.color._r, self.color._g, self.color._b)
        hsv = colorsys.rgb_to_hsv(*rgb)
        rgb = colorsys.hsv_to_rgb(val, hsv[1], hsv[2])
        self.color._r, self.color._g, self.color._b = rgb

    @property
    def s(self):
        """Saturation component as a float value"""
        rgb = (self.color._r, self.color._g, self.color._b)
        hsv = colorsys.rgb_to_hsv(*rgb)
        return hsv[1]

    @s.setter
    def s(self, val: float):
        utils.validate(val)
        rgb = (self.color._r, self.color._g, self.color._b)
        hsv = colorsys.rgb_to_hsv(*rgb)
        rgb = colorsys.hsv_to_rgb(hsv[0], val, hsv[2])
        self.color._r, self.color._g, self.color._b = rgb

    @property
    def v(self):
        """Value component as a float value"""
        rgb = (self.color._r, self.color._g, self.color._b)
        hsv = colorsys.rgb_to_hsv(*rgb)
        return hsv[2]

    @v.setter
    def v(self, val: float):
        utils.validate(val)
        rgb = (self.color._r, self.color._g, self.color._b)
        hsv = colorsys.rgb_to_hsv(*rgb)
        rgb = colorsys.hsv_to_rgb(hsv[0], hsv[1], val)
        self.color._r, self.color._g, self.color._b = rgb


class ColorViewHSV(BaseColorViewHSV):
    """
    HSV view of Color objects in float values
    """
    __slots__: List[str] = []

    def __iter__(self):
        hsv = colorsys.rgb_to_hsv(*self.color.rgb.vals)
        yield from hsv

    @property
    def vals(self) -> Sequence[float]:
        """
        Tuple of HSV values as floats.
        """
        return tuple(self)

    @vals.setter
    def vals(self, values: Sequence[float]):
        if len(values) != 3:
            raise ValueError("Needs exactly 3 floats")
        utils.validate(values[0])
        utils.validate(values[1])
        utils.validate(values[2])
        rgb = colorsys.hsv_to_rgb(values[0], values[1], values[2])
        self.color = cast("colors.Color", self.color)
        self.color.rgb.vals = rgb

    def replace(self, h=None, s=None, v=None) -> "colors.Color":
        """
        Create a new Color object by replacing the HSV values of the Color
        object associated with the colorview.

        Arguments:
          h: Hue component of HSV value as a float in the range [0, 0.1].
          s: Saturation component of HSV value as a float in the
             range [0, 0.1].
          v: Value component of HSV value as a float in the range [0, 0.1].

        Returns:
          Color object with modified HSV values.
        """
        hsv = list(self)
        if h is None:
            h = hsv[0]
        if s is None:
            s = hsv[1]
        if v is None:
            v = hsv[2]
        self.color = cast("colors.Color", self.color)
        return self.color.from_hsv(h, s, v)


class ColorViewHSVA(BaseColorViewHSV, BaseAlphaColorView):
    """
    HSVA view of Color objects in float values
    """
    __slots__: List[str] = []

    def __iter__(self):
        yield from [self.h, self.s, self.v, self.a]

    @property
    def vals(self) -> Sequence[float]:
        """
        Tuple of HSVA values as floats.
        """
        return tuple(self)

    @vals.setter
    def vals(self, values: Sequence[float]):
        if len(values) != 4:
            raise ValueError("Needs exactly 4 floats")
        utils.validate(values[0])
        utils.validate(values[1])
        utils.validate(values[2])
        utils.validate(values[3])
        rgb = colorsys.hsv_to_rgb(values[0], values[1], values[2])
        self.color = cast("colors.AlphaColor", self.color)
        self.color.rgba.vals = rgb + (values[3], )

    def replace(self, h=None, s=None, v=None, a=None) -> "colors.AlphaColor":
        """
        Create a new AlphaColor object by replacing the HSVA values of the
        AlphaColor object associated with the colorview.

        Arguments:
          h: Hue component of HSVA value as a float in the range [0, 0.1].
          s: Saturation component of HSVA value as a float in the
             range [0, 0.1].
          v: Value component of HSVA value as a float in the
             range [0, 0.1].
          a: Alpha component of HSVA value as a float in the range [0, 0.1].

        Returns:
          AlphaColor object with modified HSVA values.
        """
        hsva = list(self)
        if h is None:
            h = hsva[0]
        if s is None:
            s = hsva[1]
        if v is None:
            v = hsva[2]
        if a is None:
            a = hsva[3]
        self.color = cast("colors.AlphaColor", self.color)
        return self.color.from_hsva(h, s, v, a)



import pytest
from colorviews import AlphaColor


class TestComparisons:
    def test_eq(self):
        color_a = AlphaColor.from_int(0xefab332)
        color_b = AlphaColor.from_int(0xefab332)
        assert color_a == color_b

    def test_ne(self):
        color_a = AlphaColor.from_int(0xffab322)
        color_b = AlphaColor.from_int(0xffab323)
        assert color_a != color_b

    def test_lt(self):
        color_a = AlphaColor.from_int(0xefab380)
        color_b = AlphaColor.from_int(0xffab3ef)
        assert color_a < color_b

    def test_le(self):
        color_a = AlphaColor.from_int(0xefab332)
        color_b = AlphaColor.from_int(0xefab332)
        assert color_a <= color_b

    def test_gt(self):
        color_a = AlphaColor.from_int(0xffab332)
        color_b = AlphaColor.from_int(0xefab332)
        assert color_a > color_b

    def test_ge(self):
        color_a = AlphaColor.from_int(0xefab321)
        color_b = AlphaColor.from_int(0xefab321)
        assert color_a >= color_b


class TestFromInt:
    @pytest.mark.parametrize("colorint, rgba", [
        # can't get exact 0.5 for alpha here
        (0x5b734a80, [0.3569, 0.4510, 0.2902, 0.502]),

        (0xbf0f4abf, [0.7490, 0.0588, 0.2902, 0.749]),
    ])
    def test_valid(self, colorint, rgba):
        color = AlphaColor.from_int(colorint)
        rounded_comps = [round(comp, 4) for comp in color.rgba]
        assert rounded_comps == rgba

    @pytest.mark.parametrize("colorint", [
        0xa1fabcdef,
        -234,
    ])
    def test_invalid(self, colorint):
        with pytest.raises(ValueError):
            AlphaColor.from_int(colorint)


class TestFromRGBA:
    def test_valid(self):
        assert int(AlphaColor.from_rgba(0.6, 0.2, 0.8, 0.8)) == 0x9933cccc

    def test_invalid(self):
        with pytest.raises(ValueError):
            AlphaColor.from_rgba(0.6, 0.2, 0xcc, 0.1)


class TestFromHSLA:
    def test_valid(self):
        assert int(AlphaColor.from_hsla(0.625, 0.15, 0.74, 0.8)) == 0xb3b8c7cc

    def test_invalid(self):
        with pytest.raises(ValueError):
            AlphaColor.from_hsla(25, 1.0, 0.74, 0.8)


class TestFromHSVA:
    def test_valid(self):
        assert int(AlphaColor.from_hsva(0.625, 0.15, 0.74, 0.8)) == 0xa0a7bdcc

    def test_invalid(self):
        with pytest.raises(ValueError):
            AlphaColor.from_hsva(25, 1.0, 0.74, 0.8)


def test_hex():
    color = AlphaColor.from_int(0xabcdef12)
    assert hex(color) == "0xabcdef12"


@pytest.mark.parametrize("name, a, expected", [
    ("gray", 0.0, 0x80808000),
    ("red", 0.4, 0xff000066),
])
def test_from_name(name, a, expected):
    color = AlphaColor.from_name(name, a)
    assert int(color) == expected


def test_copy():
    orig = AlphaColor.from_hsla(0.2583, 0.1515, 0.7412, 0.25)
    copy = orig.copy()
    assert id(orig) != id(copy)
    assert orig.rgba.vals == copy.rgba.vals



_.from_name  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/colors.py:102)
_.copy  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/colors.py:205)
_.from_name  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/colors.py:251)
_.copy  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/colors.py:367)
colors  # unused import (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:23)
_.replace  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:117)
_.replace  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:166)
_.replace  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:272)
_.replace  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:325)
_.replace  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:434)
_.replace  # unused method (/home/famubu/gits/colours/pack7/colorviews/src/colorviews/views.py:487)



import pytest

from colorviews import AlphaColor


class TestGetAttr:
    @pytest.mark.parametrize("attr, expected", [
        ("h", 0.75),
        ("s", 0.47),
        ("l", 0.29),
        ("a", 0.79),
    ])
    def test_valid(self, attr, expected):
        color = AlphaColor.from_hsla(0.75, 0.47, 0.29, 0.79)
        assert round(getattr(color.hsla, attr), 4) == expected

    @pytest.mark.parametrize("attr", [
        "r", "b",
    ])
    def test_invalid(self, attr):
        color = AlphaColor.from_hsla(0.75, 0.47, 0.29, 0.79)
        with pytest.raises(AttributeError):
            getattr(color.hsla, attr)


class TestSetAttr:
    @pytest.mark.parametrize("attr, val", [
        ("h", 0.75),
        ("s", 0.5),
        ("l", 0.29),
        ("a", 0.49),
    ])
    def test_valid(self, attr, val):
        color = AlphaColor.from_hsla(0.45, 0.15, 0.89, 0.79)
        setattr(color.hsla, attr, val)
        assert round(getattr(color.hsla, attr), 4) == val

    @pytest.mark.parametrize("attr", [
        "r", "g",
    ])
    def test_invalid(self, attr):
        color = AlphaColor.from_hsla(0.75, 0.47, 0.29, 0.79)
        with pytest.raises(AttributeError):
            setattr(color.hsla, attr, 0.1)


@pytest.mark.parametrize("hsla_dict, expected", [
    ({"h": 91 / 360}, 0x496b2980),
    ({"s": 0.15}, 0x4a3f5580),
    ({"l": 0.74}, 0xbd9fdb80),
    ({"a": 0.80}, 0x4a296bcc),
    ({"h": 91 / 360, "s": 0.15}, 0x4a553f80),
    ({"h": 91 / 360, "l": 0.74}, 0xbcdb9f80),
    ({"h": 91 / 360, "l": 0.74, "a": 0.25}, 0xbcdb9f40),
    ({"s": 0.15, "l": 0.74}, 0xbdb3c780),
    ({"h": 91 / 360, "s": 0.15, "l": 0.74}, 0xbcc7b380),
])
def test_replace(hsla_dict, expected):
    color = AlphaColor.from_hsla(0.75, 0.45, 0.29, 0.5)
    assert int(color.hsla.replace(**hsla_dict)) == expected


class TestVals:
    @pytest.mark.parametrize("vals", [
        [0.2, 0.4, 0.6, 0.1],
        (0.6, 0.2, 0.4, 0.54),
    ])
    def test_setter_valid(self, vals):
        color = AlphaColor.from_hsla(0.75, 0.45, 0.29, 0.79)
        color.hsla.vals = vals
        assert [round(val, 4) for val in color.hsla] == list(vals)

    @pytest.mark.parametrize("wrong_vals", [
        [0.2, 0.4],
        (1.6, 0.2, 0.4),
        (0.6, 0.2, 0.4, 1.0, 0.8),
    ])
    def test_setter_invalid(self, wrong_vals):
        color = AlphaColor.from_hsla(0.75, 0.45, 0.29, 0.79)
        with pytest.raises(ValueError):
            color.hsla.vals = wrong_vals


def test_vals_getter():
    vals = (0.75, 0.45, 0.29, 0.79)
    color = AlphaColor.from_hsla(0.75, 0.45, 0.29, 0.79)
    assert [round(val, 4) for val in color.hsla.vals] == list(vals)
